/**
 * @apicart/vue-components v1.0.0-alpha7 
 * (c) 2018-2020 Apicart Company
 * Released under the MIT License.
 */

import Apicart from '@apicart/core-sdk';
import { Translator } from '../..';
import { ApicartButton, ApicartDialog } from './..';

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

function __awaiter(thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

function __generator(thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
}

var paymentsProcessors;
var store;
var translator = Apicart.VueComponentsTranslator || Translator;
var i18n = translator.getI18n();
var windowIsDefined = typeof window !== 'undefined';
var script = {
    name: 'apicart-order-dialog',
    i18n: i18n,
    components: {
        'apicart-button': windowIsDefined && window.ApicartButton ? window.ApicartButton : ApicartButton,
        'apicart-dialog': windowIsDefined && window.ApicartDialog ? window.ApicartDialog : ApicartDialog
    },
    data: function () {
        return {
            title: '',
            text: '',
            variableSymbol: null,
            totalPrice: null,
            order: null,
            showPayButton: false,
            paymentError: false,
            payButtonText: null,
            dialogType: 'order',
            processingPayment: false,
            showCloseButton: true,
            selectedPaymentProcessorId: null
        };
    },
    created: function () {
        paymentsProcessors = Apicart.getConfigParameter('paymentsProcessors');
        store = Apicart.getConfigParameter('store');
    },
    mounted: function () {
        var _this = this;
        var orderVariableSymbolParameterFromUrlMatch = window.location.search.match(/orderVariableSymbol=(\d+)/);
        var orderTotalPriceParameterFromUrlMatch = window.location.search.match(/orderPrice=(\d+)/);
        var orderStateParameterFromUrlMatch = window.location.search.match(/orderResult=(success|error)/);
        var orderVariableSymbolParameterFromUrl = orderVariableSymbolParameterFromUrlMatch
            ? orderVariableSymbolParameterFromUrlMatch[1]
            : null;
        var orderTotalPriceParameterFromUrl = orderTotalPriceParameterFromUrlMatch
            ? orderTotalPriceParameterFromUrlMatch[1]
            : null;
        var orderResultParameterFromUrl = orderStateParameterFromUrlMatch
            ? orderStateParameterFromUrlMatch[1]
            : null;
        if (orderResultParameterFromUrl) {
            if (orderResultParameterFromUrl === 'success') {
                this.variableSymbol = orderVariableSymbolParameterFromUrl;
                this.totalPrice = orderTotalPriceParameterFromUrl;
                this.openPaymentSuccess();
            }
            else {
                this.openPaymentError();
            }
        }
        Apicart.Utils.EventDispatcher.addListener('apicart-order-dialog-cart-finished' + this._uid, 'apicart:cart:finished', function (order) { return __awaiter(_this, void 0, void 0, function () {
            var orderCart, orderCartEntity, orderCartPaymentMethodUid, _a, _b;
            return __generator(this, function (_c) {
                switch (_c.label) {
                    case 0:
                        if (!order) {
                            this.openError();
                            return [2];
                        }
                        this.order = order;
                        return [4, order.getCart()];
                    case 1:
                        orderCart = _c.sent();
                        return [4, orderCart.getEntity()];
                    case 2:
                        orderCartEntity = _c.sent();
                        orderCartPaymentMethodUid = orderCartEntity.getPaymentMethod().getUid();
                        _a = this;
                        return [4, order.getVariableSymbol()];
                    case 3:
                        _a.variableSymbol = _c.sent();
                        _b = this;
                        return [4, orderCart.getTotalPrice()];
                    case 4:
                        _b.totalPrice = _c.sent();
                        this.selectedPaymentProcessorId = paymentsProcessors
                            && typeof paymentsProcessors[orderCartPaymentMethodUid] === 'function'
                            ? orderCartPaymentMethodUid
                            : null;
                        this.showPayButton = this.selectedPaymentProcessorId !== null;
                        this.openOrderSuccess();
                        return [2];
                }
            });
        }); });
        Apicart.Utils.EventDispatcher.addListener('apicart-order-dialog-payment-finished' + this._uid, 'apicart:payment:finished', function (isSuccess, order, info) {
            if (info === void 0) { info = {}; }
            return __awaiter(_this, void 0, void 0, function () {
                var orderCart, totalPrice, variableSymbol;
                return __generator(this, function (_a) {
                    switch (_a.label) {
                        case 0:
                            this.showPayButton = false;
                            if (!isSuccess) return [3, 5];
                            if (!(!this.totalPrice || !this.variableSymbol)) return [3, 4];
                            return [4, order.getCart()];
                        case 1:
                            orderCart = _a.sent();
                            return [4, orderCart.getTotalPrice()];
                        case 2:
                            totalPrice = _a.sent();
                            return [4, order.getVariableSymbol()];
                        case 3:
                            variableSymbol = _a.sent();
                            _a.label = 4;
                        case 4:
                            this.openPaymentSuccess();
                            return [3, 6];
                        case 5:
                            this.openPaymentError(info);
                            _a.label = 6;
                        case 6: return [2];
                    }
                });
            });
        });
    },
    methods: {
        openOrderSuccess: function () {
            this.title = this.$t('orderDialog.order.success.title');
            this.text = this.$t('orderDialog.order.success.text', {
                variableSymbol: this.variableSymbol,
                totalPrice: this.$n(this.totalPrice, 'currency')
            });
            this.dialogType = 'order';
            this.$refs.apicartDialog.open();
            this.$emit('open');
        },
        openOrderError: function () {
            this.title = this.$t('orderDialog.order.error.title');
            this.text = this.$t('orderDialog.order.error.text');
            this.dialogType = 'order';
            this.$refs.apicartDialog.open();
            this.$emit('open');
        },
        openPaymentSuccess: function () {
            this.title = this.$t('orderDialog.payment.success.title');
            this.text = this.$t('orderDialog.payment.success.text', {
                variableSymbol: this.variableSymbol,
                totalPrice: this.$n(this.totalPrice, 'currency')
            });
            this.dialogType = 'payment';
            this.$refs.apicartDialog.open();
            window.history.pushState({}, document.title, window.location.pathname);
            this.$emit('open');
        },
        openPaymentError: function (info) {
            if (info === void 0) { info = {}; }
            this.title = this.$t('orderDialog.payment.error.title');
            this.text = this.$t('orderDialog.payment.error.text', { error: info.description || 'Error' });
            this.dialogType = 'payment';
            this.showPayButton = false;
            this.paymentError = true;
            this.$refs.apicartDialog.open();
            this.$emit('open');
        },
        processPayment: function (selectedPaymentProcessorId) {
            this.close();
            paymentsProcessors[selectedPaymentProcessorId](this.order);
        },
        redirect: function () {
            window.location.href = window.location.href.split('#')[0];
        },
        close: function () {
            this.$refs.apicartDialog.close();
        }
    }
};

function styleInject(css, ref) {
  if ( ref === void 0 ) ref = {};
  var insertAt = ref.insertAt;

  if (!css || typeof document === 'undefined') { return; }

  var head = document.head || document.getElementsByTagName('head')[0];
  var style = document.createElement('style');
  style.type = 'text/css';

  if (insertAt === 'top') {
    if (head.firstChild) {
      head.insertBefore(style, head.firstChild);
    } else {
      head.appendChild(style);
    }
  } else {
    head.appendChild(style);
  }

  if (style.styleSheet) {
    style.styleSheet.cssText = css;
  } else {
    style.appendChild(document.createTextNode(css));
  }
}

var css_248z = ".apicart-order-dialog.apicart-dialog__mask{z-index:9999}.apicart-order-dialog .apicart-dialog__container{max-width:500px}";
styleInject(css_248z);

function normalizeComponent(template, style, script, scopeId, isFunctionalTemplate, moduleIdentifier /* server only */, shadowMode, createInjector, createInjectorSSR, createInjectorShadow) {
    if (typeof shadowMode !== 'boolean') {
        createInjectorSSR = createInjector;
        createInjector = shadowMode;
        shadowMode = false;
    }
    // Vue.extend constructor export interop.
    const options = typeof script === 'function' ? script.options : script;
    // render functions
    if (template && template.render) {
        options.render = template.render;
        options.staticRenderFns = template.staticRenderFns;
        options._compiled = true;
        // functional template
        if (isFunctionalTemplate) {
            options.functional = true;
        }
    }
    // scopedId
    if (scopeId) {
        options._scopeId = scopeId;
    }
    let hook;
    if (moduleIdentifier) {
        // server build
        hook = function (context) {
            // 2.3 injection
            context =
                context || // cached call
                    (this.$vnode && this.$vnode.ssrContext) || // stateful
                    (this.parent && this.parent.$vnode && this.parent.$vnode.ssrContext); // functional
            // 2.2 with runInNewContext: true
            if (!context && typeof __VUE_SSR_CONTEXT__ !== 'undefined') {
                context = __VUE_SSR_CONTEXT__;
            }
            // inject component styles
            if (style) {
                style.call(this, createInjectorSSR(context));
            }
            // register component module identifier for async chunk inference
            if (context && context._registeredComponents) {
                context._registeredComponents.add(moduleIdentifier);
            }
        };
        // used by ssr in case component is cached and beforeCreate
        // never gets called
        options._ssrRegister = hook;
    }
    else if (style) {
        hook = shadowMode
            ? function (context) {
                style.call(this, createInjectorShadow(context, this.$root.$options.shadowRoot));
            }
            : function (context) {
                style.call(this, createInjector(context));
            };
    }
    if (hook) {
        if (options.functional) {
            // register for functional component in vue file
            const originalRender = options.render;
            options.render = function renderWithStyleInjection(h, context) {
                hook.call(context);
                return originalRender(h, context);
            };
        }
        else {
            // inject component registration as beforeCreate hook
            const existing = options.beforeCreate;
            options.beforeCreate = existing ? [].concat(existing, hook) : [hook];
        }
    }
    return script;
}

/* script */
const __vue_script__ = script;
/* template */
var __vue_render__ = function () {var _vm=this;var _h=_vm.$createElement;var _c=_vm._self._c||_h;return _c('apicart-dialog',{ref:"apicartDialog",staticClass:"apicart-order-dialog",attrs:{"id":this.$t('orderDialog.id')}},[_c('h1',{attrs:{"slot":"header"},slot:"header"},[_vm._v(_vm._s(_vm.title))]),_c('div',{attrs:{"slot":"body"},domProps:{"innerHTML":_vm._s(_vm.text)},slot:"body"}),_c('div',{attrs:{"slot":"footer-buttons"},slot:"footer-buttons"},[(_vm.showPayButton)?_c('apicart-button',{attrs:{"type":"outlined"},nativeOn:{"click":function($event){return _vm.processPayment(_vm.selectedPaymentProcessorId)}}},[_vm._v(_vm._s(this.$t('orderDialog.closeButton.pay')))]):_c('apicart-button',{attrs:{"type":"outlined"},nativeOn:{"click":function($event){return _vm.redirect($event)}}},[(_vm.paymentError)?_c('span',[_vm._v(_vm._s(this.$t('orderDialog.closeButton.close')))]):_c('span',[_vm._v(_vm._s(this.$t('orderDialog.closeButton.ok')))])])],1)])};
var __vue_staticRenderFns__ = [];

  /* style */
  const __vue_inject_styles__ = undefined;
  /* scoped */
  const __vue_scope_id__ = undefined;
  /* module identifier */
  const __vue_module_identifier__ = undefined;
  /* functional template */
  const __vue_is_functional_template__ = false;
  /* style inject */
  
  /* style inject SSR */
  
  /* style inject shadow dom */
  

  
  const __vue_component__ = normalizeComponent(
    { render: __vue_render__, staticRenderFns: __vue_staticRenderFns__ },
    __vue_inject_styles__,
    __vue_script__,
    __vue_scope_id__,
    __vue_is_functional_template__,
    __vue_module_identifier__,
    false,
    undefined,
    undefined,
    undefined
  );

export default __vue_component__;
