/**
 * @apicart/vue-components v1.0.0-alpha7 
 * (c) 2018-2020 Apicart Company
 * Released under the MIT License.
 */

import Apicart from '@apicart/core-sdk';

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

function __awaiter(thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

function __generator(thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
}

var ProductsDownloader = (function () {
    function ProductsDownloader() {
        this._localStorage = Apicart.Storage;
        this._storageName = 'apicart-products-files';
        this._loadedProducts = {};
        this._loadedProducts = this.getStorage();
    }
    ProductsDownloader.prototype.getStorageName = function () {
        return this._storageName;
    };
    ProductsDownloader.prototype.getStorage = function () {
        return this._localStorage.getItem(this.getStorageName()) || {};
    };
    ProductsDownloader.prototype.getLoadedProducts = function () {
        return this._loadedProducts;
    };
    ProductsDownloader.prototype.getLoadedProductsData = function (productsFilesConfigOrUrl) {
        var _this = this;
        var products = [];
        Apicart.Utils.Loops
            .forEach(productsFilesConfigOrUrl, function (productDataConfigOrUrl) {
            var dataUrl = productDataConfigOrUrl;
            var pageUrl = null;
            var data = null;
            if (typeof productDataConfigOrUrl !== 'string') {
                dataUrl = productDataConfigOrUrl.dataUrl;
                pageUrl = productDataConfigOrUrl.pageUrl;
                if (productDataConfigOrUrl.data && Object.keys(productDataConfigOrUrl.data).length) {
                    data = productDataConfigOrUrl.data;
                }
            }
            if (!data && !(dataUrl in _this._loadedProducts)) {
                return;
            }
            products.push({
                dataUrl: dataUrl,
                data: data || _this._loadedProducts[dataUrl].data,
                pageUrl: pageUrl || _this._loadedProducts[dataUrl].pageUrl
            });
        });
        return products;
    };
    ProductsDownloader.prototype.getProductsData = function (productsFilesConfigOrUrl) {
        return __awaiter(this, void 0, void 0, function () {
            var localStorage, productsPromises, products, filteredProducts;
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        localStorage = Apicart.Storage;
                        productsPromises = [];
                        Apicart.Utils.Loops
                            .forEach(productsFilesConfigOrUrl, function (productDataConfigOrUrl) {
                            var dataUrl = productDataConfigOrUrl;
                            var pageUrl = null;
                            var data = null;
                            if (typeof productDataConfigOrUrl !== 'string') {
                                dataUrl = productDataConfigOrUrl.dataUrl;
                                pageUrl = productDataConfigOrUrl.pageUrl;
                                data = productDataConfigOrUrl.data || null;
                            }
                            var dataNotEmpty = data && Object.keys(data).length !== 0;
                            if (dataUrl in _this._loadedProducts || dataNotEmpty) {
                                productsPromises.push(Promise.resolve({
                                    dataUrl: dataUrl,
                                    data: dataNotEmpty ? data : _this._loadedProducts[dataUrl].data,
                                    pageUrl: pageUrl
                                }));
                            }
                            else {
                                productsPromises.push((function () { return __awaiter(_this, void 0, void 0, function () {
                                    var productDataResponse;
                                    return __generator(this, function (_a) {
                                        switch (_a.label) {
                                            case 0: return [4, Apicart.Utils.Ajax.get(dataUrl)];
                                            case 1:
                                                productDataResponse = _a.sent();
                                                if (typeof productDataResponse !== 'object' || parseInt(productDataResponse.status) !== 200) {
                                                    return [2];
                                                }
                                                return [2, {
                                                        dataUrl: dataUrl,
                                                        data: productDataResponse.data,
                                                        pageUrl: pageUrl
                                                    }];
                                        }
                                    });
                                }); })());
                            }
                        });
                        return [4, Promise.all(productsPromises)];
                    case 1:
                        products = _a.sent();
                        filteredProducts = [];
                        Apicart.Utils.Loops.forEach(products, function (product) {
                            if (!product) {
                                return;
                            }
                            _this._loadedProducts[product.dataUrl] = {
                                data: product.data,
                                pageUrl: product.pageUrl
                            };
                            filteredProducts.push(product);
                        });
                        localStorage.setItem(this._storageName, this._loadedProducts);
                        return [2, filteredProducts];
                }
            });
        });
    };
    ProductsDownloader.prototype.getProductData = function (productUrl) {
        return __awaiter(this, void 0, void 0, function () {
            var response;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4, this.getProductsData([productUrl])];
                    case 1:
                        response = _a.sent();
                        return [2, response.length === 0 || typeof response[0] === 'undefined' ? null : response[0].data];
                }
            });
        });
    };
    ProductsDownloader.prototype.getLoadedProductData = function (productUrl) {
        var products = this.getLoadedProductsData([productUrl]);
        return products.length === 1 ? products[0] : null;
    };
    return ProductsDownloader;
}());
var ProductsDownloader$1 = new ProductsDownloader();

export default ProductsDownloader$1;
